c==================================================================================
c     mu2chi Version 2.1 (Feburary 2018)
c==================================================================================
c       changes from v2.0: 
c         - Auto detection of rising post-edge data with flatten option
c         - Minor bug fix re feeding of pre-edge fit range parameters
c----------------------------------------------------------------------------------
c
c     XAFS Data Reduction Tool utilizing ifeffit
c     Written by:  Martin J. Schalken, Christopher T. Chantler
c     (Based on origninal code by: Grant M. Schuster)
c
c     Features:
c       - Choice of Non-interpolated (recommended), Cubic, or Linear interpolation 
c       - Preservation of information content for Cubic interpolation via interval-wise 
c          scaling of uncertainties
c       - If erroneous pre-edge fit detected, will skip preedge subtraction
c       - Ability to automatically determine k-spacing
c       - Option to output intervals where information lost
c       - Will detect artificial rising spectrum trend, and with option to 'flatten'
c
c----------------------------------------------------------------------------------
c     This program relies on ifeffit version 1.2.3 or later.
c     This program uses GNUPLOT to generate plots
c
c     To execute this program compile using:
c     'make' with the Makefile configured according to the
c     ifeffit documentation. 
c
c     Refer to mu2chi manual for more details
c===================================================================================

      implicit none
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      integer i, j, m, n, p, q, r, s, l, nlines
      integer first_energy,kpoints, kpoints_int, length
      integer failsafe, ptsinrange, intn

      real*8 kmin,kmax,kspace,dk,nknots,rbkg
      real*8 e0, kweight, pre2, pre1
      real*8 fit_min, fit_max
      real*8 u0_intn

      real*8 my_energy(1000),my_chik(1000),my_k(1000)
      real*8 my_pre(1000),my_bkg(1000),my_preedge(1000)
      real*8 my_final(1000)
      real*8 my_pre_flat(1000), my_bkg_flat(1000)
      real*8 my_dchik(1000),my_dxmu(1000), my_xmu(1000)
      real*8 interp_k(8000),interp_chik(8000), interp_dchik(8000)
      real*8 nist_energy(1000), nist_bkg(1000)

      character*1 interpolation, option
      character*99 outputname,inputfile
      character*128 arg,formt
      character*99 param, value, input

      logical err_cor, dispinfloss, plots, auto_e0, flattened

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



c     Tell the program where to find the ifeffit library file:

      include 'ifeffit.inc'

c     Initialise ifeffit and
c     Set up for screen echo so that we can see what is being
c     reported in IFEFFIT:
      i = ifeffit(' ')
      i = ifeffit('reset')
      i = ifeffit('&screen_echo=0')
 


      kmin=0.d0
      kmax=1.4d1
      dk=3.d-1 

   
c--------------------------------------------------------------------
c START
c--------------------------------------------------------------------

c     DEFINE DEFAULTS
       interpolation = 'N'
       err_cor = .true.
       kspace = 0.0d-2
       fit_min = 3.0
       fit_max = 10.0
       nknots = 7
       rbkg = 1.0
       auto_e0=.true.
       kweight=2
       pre1 = -10
       pre2 = -30
       plots = .true.
       dispinfloss = .true.
       outputname='mu2chi_out'

       length=0

c     DETERMINE LENGTH OF INPUT FILE
       nlines = 0
       open (unit=1, file = 'input_params.txt')
       do i=1,1000,1
       read (1,*, END=10) input
       if(input(1:1).ne.' ') nlines = nlines + 1
       end do
 10    close (1) 


c     READ PARAMETER FILE
       open(unit=1, file='input_params.txt')
       
       value = ' '
       read(1,'(A)') input
       if(input(1:9) .eq. 'inputfile') then
       value = input(11:99)
       length=0
       do j=1,99,1
       if(value(j:j).NE.' ')length=length+1
       end do
       if(length.LE.9)
     &  write(formt,'(A,I1,A)')"(A,A",length,",A)"
       if(length.GT.9)
     &  write(formt,'(A,I2,A)')"(A,A",length,",A)"

       write(arg,formt)'read_data(',value,
     &',group=mudata, type=xmu)'
      i = ifeffit(arg) 


       else
       write(*,*)"ERROR: input file must be specified" 
       write(*,*)"in first line of 'input_params.txt'"
       goto 1005
       end if

       do i=2,nlines,1
       value = ' '
       read(1,*)param, value
       if(param(1:6) .eq. 'output') outputname = value
       if(param .eq. 'interpolation') interpolation = value(1:1)
       if(param .eq. 'error_preservation') then 
         if(value(1:1).eq.'t') err_cor = .true.
         if(value(1:1).eq.'f') err_cor = .false.
       endif
       if(param .eq. 'kspace') read(value,*) kspace 
       if(param .eq. 'fit_min') read(value,*) fit_min 
       if(param .eq. 'fit_max') read(value,*) fit_max 
       if(param .eq. 'nknots') read(value,*) nknots  
       if(param .eq. 'rbkg') read(value,*) rbkg 
       if(param .eq. 'e0') then 
         read(value,*) e0 
         auto_e0=.false.
       end if
       if(param .eq. 'kweight') read(value,*) kweight 
       if(param .eq. 'pre1') read(value,*) pre1 
       if(param .eq. 'pre2') read(value,*) pre2 
       if(param(1:7) .eq. 'display') then 
         if(value(1:1).eq.'t') dispinfloss = .true.
         if(value(1:1).eq.'f') dispinfloss = .false.
       end if
       if(param(1:5) .eq. 'plots') then 
         if(value(1:1).eq.'t') plots = .true.
         if(value(1:1).eq.'f') plots = .false.
       end if
       end do  
       close(1)




c===============================================================
c===============================================================


c     Bit of string formatting/preparation

      length=0
      do i=1,99,1
      if(outputname(i:i).NE.' ')length=length+1
      end do
      if(length.LE.9)
     & write(formt,'(A,I1,A)')"(A,A",length,",A)"
      if(length.GT.9)
     & write(formt,'(A,I2,A)')"(A,A",length,",A)"


      i = iffputsca('i_nknots',nknots)
      i = iffputsca('i_rbkg',rbkg)
      i = iffputsca('e0',e0)

      i = iffputsca('pre2',pre2)
      i = iffputsca('pre1',pre1)

      i = iffputsca('i_kmin',kmin)
      i = ifeffit('set(kmin=i_kmin)')
      i = iffputsca('i_kmax',kmax)
      i = ifeffit('set(kmax=i_kmax)')
      i = iffputsca('i_dk',dk)
      i = ifeffit('set(dk=i_dk)')

      i = iffputsca('kweight',kweight)
      i = ifeffit('i_kweight=kweight')

      i = ifeffit('mudata.dxmu=mudata.3')
   



c--------------------------------------------------------------------
c START PRE_EDGE CALC
c--------------------------------------------------------------------
c     sets mudata.pre to be the pre-edge subtracted xmu

      if(auto_e0)then
      i = ifeffit('pre_edge(mudata.energy,mudata.xmu,
     &               pre2=pre2,pre1=pre1,find_e0=true)') 
      write(*,*)'Will automatically determine e0'
      else
      i = ifeffit('pre_edge(mudata.energy,mudata.xmu,
     &                e0=e0)') 
      end if

c-------------------------------------------------------------------
c END PRE_EDGE CALC
c--------------------------------------------------------------------

c     Now spline the pre_edge_subtracted data.

      i = ifeffit('&e0_guess=e0')

 1001 continue

c     sets mudata.bkg
      i = ifeffit('spline(mudata.energy, mudata.pre,
     &     kweight=kweight, e0=e0, nknots=i_nknots,
     &        rbkg=i_rbkg)')
c--------------------------------------------------------------------

      i = ifeffit('mudata.preedge=mudata.xmu-mudata.pre')
      i = ifeffit('mudata.final=mudata.pre-mudata.bkg')


      m = iffgetarr('mudata.pre',my_pre) !after pre-edge subtraction
      p = iffgetarr('mudata.bkg',my_bkg)  !background spline
      n = iffgetarr('mudata.energy',my_energy) !energies
      q = iffgetarr('mudata.dxmu',my_dxmu) !uncertainties
      r = iffgetarr('mudata.xmu',my_xmu) !before pre-edge subtraction
      s = iffgetarr('mudata.preedge',my_preedge) !pre-edge
      s = iffgetarr('mudata.final',my_final) !final, background subtracted


 1002 continue


      i = iffgetsca('e0',e0)
      flattened=.false.

      j=1
      do while (my_energy(j).LT.e0)
         j=j+1
      end do
      first_energy=j-1


      do l=j+1,p,1
c      write(*,*)my_preedge(l), my_bkg(l)
       if(my_bkg(l).LT.0.0)then
      write(*,*)'--------------------------'
       write(*,*)'WARNING: Error in bkg calculation.'
       write(*,*)'Possibly insufficient pre-edge points to fit preedge.'
       write(*,*)'Will not subtract pre-edge.'
      write(*,*)'--------------------------'
c      Replace pre-edge subtracted spectra with original xmu data 
        i = ifeffit ('mudata.pre=mudata.xmu')
        goto 1001
       end if
      end do


      
       if(my_bkg(j+50).LT.my_bkg(j+60))then

      write(*,*)'--------------------------'
       write(*,*)'WARNING: Increasing spectrum trend detected.'
       write(*,*)''
       write(*,*)'Proceed with Flattening? (y/n)'
       read(*,'(A1)')option
       if(option.eq.'y') then
        flattened=.true.
        write(*,*)'Will flatten post edge data'
       else
         write(*,*)'Will not flatten post edge data'
       end if
      write(*,*)'--------------------------'
      
        end if

       if(flattened)then
c     Use intersection with bkg to determine flattening starting point
      do l=j,p,1
       if(((my_bkg(l+1).LT.my_pre(l+1))
     & .AND.(my_bkg(l).GT.my_pre(l))).OR.
     & ((my_bkg(l).LT.my_pre(l)).AND.
     & (my_bkg(l+1).GT.my_pre(l+1))))then 
             u0_intn=0.5*(my_bkg(l)+my_bkg(l+1))
             intn=l
      goto 1003
         end if
        end do

 1003  continue

c       Flatten post edge spectrum
        do l=1,p,1
         if(l.LT.intn)then
         my_pre_flat(l)=my_pre(l)
         my_bkg_flat(l)=my_bkg(l)
         else
         my_pre_flat(l)=my_pre(l)*u0_intn/my_bkg(l)
         my_bkg_flat(l)=u0_intn
         end if
        end do

       end if

 1004  continue
    


      do while (j.LE.m)

         my_k(j-first_energy)=(1.0d-10*SQRT(2*1.6d-19*9.11d-31*
     &        (my_energy(j)-e0))/1.055d-34)
     
      if(flattened)then
         my_chik(j-first_energy)=(my_pre_flat(j)-my_bkg_flat(j))/
     &        my_bkg_flat(j)
   
         my_dchik(j-first_energy)=my_dxmu(j)/my_bkg_flat(j)

       else
         my_chik(j-first_energy)=(my_pre(j)-my_bkg(j))/
     &        my_bkg(j)
   
         my_dchik(j-first_energy)=my_dxmu(j)/my_bkg(j)
       end if

      j=j+1

      end do

      kpoints=m-first_energy

c     Write supplementary data file

      open(unit=10,file='supp_data.dat')	

      if(flattened) then
      write(10,*)
     &"# Energy, Original xmu, pre-edge, pre-edge removed,
     & bkg spline, bkg removed, uncertainty, 
     & flattened pre-edge, flattened pre-edge removed"

      do i=1,p,1
      if(my_energy(i).gt.0.0) then
      write(10,"(F9.2,F8.4,F8.4,F8.4,F8.4,F8.4,F10.6,F8.4,F8.4)")
     & my_energy(i),my_xmu(i),my_preedge(i),my_pre(i),my_bkg(i),
     & my_final(i),my_dxmu(i), my_pre_flat(i),my_bkg_flat(i)
      end if
      end do

      else
      write(10,*)
     &"# Energy, Original xmu, pre-edge, pre-edge removed,
     & bkg spline, bkg removed, uncertainty"

      do i=1,p,1
      if(my_energy(i).gt.0.0) then
      write(10,*)my_energy(i),my_xmu(i),my_preedge(i),
     & my_pre(i),my_bkg(i),my_final(i),my_dxmu(i)
      end if
      end do

      end if
      close(10)

c=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

c ===== NO INTERPOLATION =====     
      if(interpolation.EQ.'N') then     
c     Put both my_chik and my_k back into IFEFFIT:
      i = iffputarr('mudata.k',   kpoints,my_k)
      i = iffputarr('mudata.chi', kpoints,my_chik)
      i = iffputarr('mudata.dchi',kpoints,my_dchik)
      i = ifeffit('mudata.e0 = ones(npts(mudata.k)) * e0')




c      WRITE DATA FILE 
       write(arg,formt)'write_data(file=',outputname,
     &'.dat,$title*, mudata.k,mudata.chi,mudata.dchi,mudata.e0)'
       i = ifeffit(arg)


      write(*,*)''
      write(*,*)''
      write(*,*)'***************************************'
      write(*,*)'******* mu2chi PROGRAM COMPLETE *******'
      write(*,*)''
      write(*,*)'No Interpolation'
      write(*,*)''

      end if


c=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

c ===== CUBIC INTERPOLATION =====
      if(interpolation(1:1).EQ.'C') then
      call mjs_cubterp(kpoints,my_k,my_chik,my_dchik,
     & kpoints_int,interp_k,interp_chik,interp_dchik, kspace,
     & err_cor, dispinfloss, fit_min, fit_max)

       i = iffputarr('mudata.k',    kpoints_int, interp_k)
       i = iffputarr('mudata.chi',  kpoints_int, interp_chik)
       i = iffputarr('mudata.dchi', kpoints_int, interp_dchik)
       i = ifeffit('mudata.e0 = ones(npts(mudata.k)) * e0')


c      WRITE DATA FILE 
       write(arg,formt)'write_data(file=',outputname,
     &'.dat,$title*, mudata.k,mudata.chi,mudata.dchi,mudata.e0)'
       i = ifeffit(arg)

      end if


c=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


c ===== LINEAR INTERPOLATION =====
      if(interpolation.EQ.'L') then
      call mjs_linterp(kpoints,my_k,my_chik,my_dchik,
     & kpoints_int,interp_k,interp_chik,interp_dchik, kspace,
     & err_cor, dispinfloss, fit_min, fit_max)  
       
       i = iffputarr('mudata.k',    kpoints_int, interp_k)
       i = iffputarr('mudata.chi',  kpoints_int, interp_chik)
       i = iffputarr('mudata.dchi', kpoints_int, interp_dchik)
       i = ifeffit('mudata.e0 = ones(npts(mudata.k)) * e0')
         
c      WRITE DATA FILE 
       write(arg,formt)'write_data(file=',outputname,
     &'.dat,$title*, mudata.k,mudata.chi,mudata.dchi,mudata.e0)'
       i = ifeffit(arg)
   
      end if

      write(*,formt)' Output written to "./',outputname,'.dat"'
      write(*,*)'Additional output written to "supp_data.dat"'
      write(*,*)''

c=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-     
 


      !call print_echo


      if (plots) then
      call plot_preedge
      call plot_bkg
      call plot_final_emu
      call plot_chivk(outputname)
      if(flattened)then 
       write(*,*)'NOTE: Flattened data not used in bkg / preedge plot.'
       write(*,*)'Please use supp_data.dat' 
       write(*,*)'to plot spectrum pre and post flattening'
      end if
      end if

      write(*,*)''
      write(*,*)'Have a nice day.'
      write(*,*)'***************************************'
      write(*,*)''

 1005 continue
      end
c==================================================================
c==================================================================

c     CUBIC INTERPOLATION SUBROUTINE

      subroutine mjs_cubterp(pointsin,xin,yin,yerrin,
     & pointsout,xout,yout,yerrout,kspace,err_cor,dispinfloss,
     & fit_min,fit_max)

      integer i,j
      integer pointsin, pointsout, pointsinwindow
      integer pointslost, ptsinrange

      real*8 kspace
      real*8 sigmasqin, sigmasqout, sigmasqdiff
      real*8 etarat, fit_min, fit_max

      real*8 pointsoutwindow(1000)
      real*8 xin(1000), yin(1000), yerrin(1000)
      real*8 xout(8000), yout(8000), yerrout(8000)

      double precision grad1,grad2,y0,y1,y2,y3,x,a,b,c,d
      double precision x1, x2

      logical dispinfloss, err_cor


      pointsout=0
      ptsinrange=0

      if(kspace .eq. 0) then !do automatic k-space
       kspace=100.0
       do i=1,pointsin-1,10
        if((xin(i+20)-xin(i))/20.0 .lt.kspace .AND. 
     &     (xin(i+20)-xin(i))/20.0 .gt. 0.01) then 
         kspace=(xin(i+20)-xin(i))/20.0
        end if
       end do
      kspace=(dble(int(kspace*100))/100.0)
      write(*,'(A,F5.3)')'k-spacing determined as: ',kspace

      end if
      
c     Make interpolated grid
      do j=1,8000,1
      xout(j)=kspace*j
      end do
      


      do j=1,8000,1 !go through each output point

c     Find enclosing non-interpolated input points
      do i=1,pointsin,  1
      if((xin(i).LT.xout(j)).AND.(xout(j).LT.xin(i+1))) then

      	if(i.EQ.1)then !do a linear interpolation in first interval
      	 grad1=(yin(i+1)-yin(i))/(xin(i+1)-xin(i))
     	 yout(j)=yin(i)+grad1*(xout(j)-xin(i))
     	 goto 2001 !skip the main interpolation calculations
     	end if

c     Calculate gradient at first point
      grad1=(yin(i+1)-yin(i-1))/(xin(i+1)-xin(i-1))
c     Calculate gradient at second point
      grad2=(yin(i+2)-yin(i))/(xin(i+2)-xin(i))

      x1=xin(i)
      x2=xin(i+1)
      y0=yin(i-1)
      y1=yin(i)
      y2=yin(i+1)
      y3=yin(i+2)
      x =xout(j)



c     Calculate Polynomial Coefficients
      a=(grad2-((y1-y2)/(x1-x2))-((grad1-grad2)/(2*(x1-x2)))*
     &(2*x2-((x1**2-x2**2)/(x1-x2)))) /
     &(3*x2**2-((x1**3-x2**3)/(x1-x2))-3*x2*(x1**2-x2**2)/(x1-x2)
     &+(3/2)*((x1**2-x2**2)**2)/((x1-x2)**2))

      b=((grad1-grad2)-3*a*(x1**2-x2**2))/(2*(x1-x2))

      c=grad1-3*a*x1**2-2*b*x1

      d=((xin(i+1)-xout(j))/(xin(i+1)-xin(i)))*
     &(y1-a*x1**3-b*x1**2-c*x1)+
     &((xout(j)-xin(i))/(xin(i+1)-xin(i)))*
     &(y2-a*x2**3-b*x2**2-c*x2)

c     y output
      yout(j)=a*x**3+b*x**2+c*x+d

 2001 continue


c     Determine error at interpolated point
      yerrout(j)=sqrt(
     &(((xin(i+1)-xout(j))/(xin(i+1)-xin(i)))**2)*   
     &(yerrin(i))**2 +
     &(((xout(j)-xin(i))/(xin(i+1)-xin(i)))**2)*
     & (yerrin(i+1))**2)

      pointsout=pointsout+1  
      
      goto 2002 !this interpolated point is done, exit i loop and move to next output point
      end if
      end do
 2002 continue

      end do

      if(.NOT. err_cor) then
c     Count how many interpolated points between i and i+1
      do i=1,pointsin,1
      pointsoutwindow(i)=0
      do j=1, pointsout, 1
      if ((xout(j).GT.xin(i)).AND.(xout(j).LE.xin(i+1))) then
       pointsoutwindow(i)=pointsoutwindow(i)+1.0
      end if
      end do
      end do
      end if
c======================================================================
c     APPLY ERROR CORRECTION TO PRESERVE INFORMATION CONTENT

      if(err_cor) then

      do i=1,pointsin,1
       if (xin(i).ge.fit_min .AND. xin(i).le.fit_max)then
        ptsinrange=ptsinrange+1
       endif
      end do

c     calculate eta ratio
      etarat=real(ptsinrange-4)/(((fit_max-fit_min)/kspace)- 4.0)

      !write(*,*)"etarat = ", etarat

      do i=1, pointsin, 1

      sigmasqout=0
      sigmasqin=0
      sigmasqdiff=0

c     Count how many interpolated points between i and i+1
      pointsoutwindow(i)=0
      do j=1, pointsout, 1
      if ((xout(j).GT.xin(i)).AND.(xout(j).LE.xin(i+1))) then
       pointsoutwindow(i)=pointsoutwindow(i)+1.0
      end if
      end do

      if(pointsoutwindow(i) .eq. 0) goto 2004
c     Determine sum of sigma sq in in the interval
      sigmasqin=0.5*(1.0/(yerrin(i)**2)+(1.0/yerrin(i+1)**2))

      beta=0.5
      failsafe=0


 487  continue
c     Determine sum of sigma sq out in the interval
      !write(*,*)"sigmasqdiff = ",sigmasqdiff
      sigmasqout=0
      do j=1, pointsout, 1
      if ((xout(j).GT.xin(i)).AND.(xout(j).LE.xin(i+1))) then
       sigmasqout=sigmasqout+(1.0/(yerrout(j)**2))
      end if
      end do


      sigmasqdiff = 0

      sigmasqdiff=((sigmasqout/sigmasqin)*etarat)
      sigmasqdiff = 1.0/sigmasqdiff 


      !write(*,*)"sigmasqin   = ",sigmasqin
      !write(*,*)"sigmasqout  = ",sigmasqout
      !write(*,*)"sigmasqdiff = ",sigmasqdiff
      !write(*,*)"etarat      = ",etarat

      if (sigmasqdiff .lt. 1.0001 .AND. sigmasqdiff .gt. 0.9999) then
       goto 2003 
      end if

      
c    Check if failsafe required
      if(failsafe .GT. 1000 .OR. pointsoutwindow(i) .EQ. 0) then 
       goto 2004
      end if

c    If too low, then increase height of curve
      if(sigmasqdiff .GT. 1.0001)then

      failsafe=failsafe+1

      !Decrease beta parameter
      beta=beta-0.001

      do j=1,pointsout,1
      if ((xout(j).GT.xin(i)).AND.(xout(j).LE.xin(i+1))) then
             yerrout(j)=(
     &(((xin(i+1)-xout(j))/(xin(i+1)-xin(i)))**(1/beta))*   
     &(yerrin(i))**2 +
     &(((xout(j)-xin(i))/(xin(i+1)-xin(i)))**(1/beta))*
     & (yerrin(i+1))**2)**0.5
      end if
      end do


      goto 487
      end if

c    If too high, decrease height of curve
      if(sigmasqdiff .LT. 0.9999)then

      failsafe=failsafe+1


      !Increase beta parameter

      beta=beta+0.001

      do j=1,pointsout,1
      if ((xout(j).GT.xin(i)).AND.(xout(j).LE.xin(i+1))) then
             yerrout(j)=(
     &(((xin(i+1)-xout(j))/(xin(i+1)-xin(i)))**(1/beta))*   
     &(yerrin(i))**2 +
     &(((xout(j)-xin(i))/(xin(i+1)-xin(i)))**(1/beta))*
     & (yerrin(i+1))**2)**0.5
      end if
      end do

      goto 487
      end if

 2003 continue
c     Print interval result
      !write(*,*)"*******************************************"
      !write(*,*)"interval no = ",i
      !write(*,*)"sigmasqdiff = ",sigmasqdiff

 2004  continue
      
      end do

      end if !end uncertainty correction


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
c      goto 2005
c     Check first few points aren't zero
       do j=2,20,1
       if(yout(j-1).eq.0.0 .AND. yout(j).ne.0.0) then
       grad=(yout(j)-yout(1))/(xout(j)-xout(1))
        do i=1,j-1,1
         yout(i)=yout(1)+xout(i)*grad
         yerrout(i)=yerrout(j)
        end do
        goto 2005
       end if
       end do
 2005  continue

c     Display information loss
      if(dispinfloss)then
      write(*,*)'--------------------------'
      pointslost=0
      do i=1,pointsin,1
       if (pointsoutwindow(i).eq.0.0) then
        pointslost=pointslost+1
        write(*,*)'Information lost at interval beginning at',
     &    xin(i)
       end if
      end do

      end if



c     Completion message
      write(*,*)''
      write(*,*)''

      if(pointslost/pointsin .GT. 0.1) then
       write(*,*)'**************************'
       write(*,*)'WARNING: More than 10% of original intervals'
       write(*,*)'contain no interpolated points.'
       write(*,*)'May affect reliability of resulting chi^2_r.'
       write(*,*)'Please consider finer k-spacing'
       write(*,*)'**************************'

      end if
      if(pointsout/pointsin .GT. 20.0) then
       write(*,*)'**************************'
       write(*,*)'WARNING: Point density has greatly increased (>20x).'
       write(*,*)'May affect reliability of resulting chi^2_r.'
       write(*,*)'Please choose greater k-spacing'
       write(*,*)'**************************'

      end if

      write(*,*)''
      write(*,*)''
      write(*,*)'***************************************'
      write(*,*)'******* mu2chi PROGRAM COMPLETE *******'
      write(*,*)''
      write(*,*)'--------------------------------------'

      if(err_cor) then
         write(*,*)'Cubic Interpolation - with error correction'
      else
         write(*,*)'Cubic Interpolation'
      end if
 
      write(*,'(A,F4.2)')' k-spacing:          ', kspace
      if(err_cor)then
      write(*,'(A,F3.1,A,F3.1)')' Fit Range:          ',
     &  fit_min,' - ',fit_max
      end if
      write(*,*)''
      write(*,'(A21,I4)')'Total Points in:     ', pointsin
      write(*,'(A21,I4)')'Total Points out:    ', pointsout
      write(*,'(A21,I3)')'Total Intervals lost:', pointslost
      write(*,*)'--------------------------------------'
      write(*,*)''


 2006 continue
      return
      end

c==================================================================
c==================================================================

      subroutine mjs_linterp(pointsin,xin,yin,yerrin,
     & pointsout,xout,yout,yerrout,kspace,err_cor,dispinfloss,
     & fit_min,fit_max)

      integer i,j,failsafe
      integer pointsin, pointsout, pointslost

      real*8 grad, kspace 
      real*8 sigmasqin, sigmasqout, sigmasqdiff
      real*8 etarat, fit_min, fit_max

      real*8 pointsoutwindow(1000)
      real*8 xin(1000), yin(1000), yerrin(1000)
      real*8 xout(8000), yout(8000), yerrout(8000)

      logical err_cor, dispinfloss


      pointsout=0

       if(kspace .eq. 0) then !do automatic k-space
       kspace=100.0
       do i=1,pointsin-1,10
        if((xin(i+20)-xin(i))/20.0 .lt.kspace .AND. 
     &     (xin(i+20)-xin(i))/20.0 .gt. 0.01) then 
         kspace=(xin(i+20)-xin(i))/20.0
        end if
       end do
      kspace=(dble(int(kspace*100))/100.0)
      write(*,'(A,F5.3)')'k-spacing determined as: ',kspace

      end if
   
c     Make interpolated grid
      do j=1,8000,1
      xout(j)=kspace*j
      end do

      do j=1, 8000, 1
      do i=1, pointsin,  1
c     Check if a point is needed between two input points
      if((xin(i).LT.xout(j)).AND.(xout(j).LT.xin(i+1))) then
   
 
c     Calculate gradient between points
      grad=(yin(i+1)-yin(i))/(xin(i+1)-xin(i))

c     Determine y value at interpolated point
      yout(j)=yin(i)+grad*(xout(j)-xin(i))

c     Determine error at interpolated point
      yerrout(j)=yerrin(i)+(yerrin(i+1)-yerrin(i))
     &   *(xout(j)-xin(i))/(xin(i+1)-xin(i))


       pointsout=pointsout+1  
       goto 3001
      end if
      end do
 3001  continue
      end do


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

c     Check first few points aren't zero
       do j=2,pointsout,1
       if(yout(1).eq.0.0 .AND. yout(j).ne.0.0) then
       grad=(yout(j)-yout(1))/(xout(j)-xout(1))
        do i=1,j-1,1
         yout(i)=yout(1)+xout(i)*grad
         yerrout(i)=yerrout(j)
        end do
        goto 3002
       end if
       end do
 3002  continue



c     Display information loss
      if(dispinfloss)then
      write(*,*)'--------------------------'
      pointslost=0
      do i=1,pointsin,1
       if (pointsoutwindow(i).eq.0.0) then
        pointslost=pointslost+1
        write(*,*)'Information lost at interval beginning at',
     &    xin(i)
       end if
      end do

      end if

c     Completion message
      write(*,*)''
      write(*,*)''
      if(pointslost/pointsin .GT. 0.20) then
       write(*,*)'**************************'
       write(*,*)'ERROR: More than 20% of original intervals'
       write(*,*)'contain no interpolated points.'
       write(*,*)'Resulting chi^2_r may be unnecessarily large'
       write(*,*)'Please choose finer k-spacing'
       write(*,*)'**************************'
c       goto 3003
      end if
      if(pointsout/pointsin .GT. 20.0) then
       write(*,*)'**************************'
       write(*,*)'ERROR: Point density has greatly increased.'
       write(*,*)'May affect reliability of resulting chi^2_r.'
       write(*,*)'Please choose greater k-spacing'
       write(*,*)'**************************'
c       goto 3003
      end if

      write(*,*)''
      write(*,*)''
      write(*,*)'***************************************'
      write(*,*)'******* mu2chi PROGRAM COMPLETE *******'
      write(*,*)''
      write(*,*)'--------------------------------------'
      write(*,*)'Linear Interpolation'
      write(*,'(A,F4.2)')' k-spacing:          ', kspace
      write(*,'(A21,I4)')'Total Points in:     ', pointsin
      write(*,'(A21,I4)')'Total Points out:    ', pointsout
      write(*,'(A21,I3)')'Total Intervals lost:', pointslost
      write(*,*)'--------------------------------------'
      write(*,*)''




 3003 continue
      return
      end

c===================================================================

      subroutine plot_preedge
      open(unit=200,file="preedge.gnu")
       write(200,*)'set terminal png enhanced size 1920,1080'
       write(200,*)"set output 'preedge.png' "
       write(200,*)'set grid lw 1.2'
       write(200,*)'set border lw 3'
       write(200,*)'unset key'
       write(200,*)'set termoption font ",36"'
       write(200,*)'set xlabel "Energy (eV)"'
       write(200,*)'set ylabel "[{/Symbol m}/{/Symbol r}]"'
       write(200,*)'plot "supp_data.dat" using 1:2 
     &    with lines lw 3 lc rgb "blue",
     &   "supp_data.dat" using 1:3 
     &    with lines lw 3 lc rgb "red" '
       write(200,*)'quit'
       close(200)

       call system('gnuplot preedge.gnu')
       call system('rm preedge.gnu')
       write(*,*)'pre-edge plotted in "preedge.png"'
       end
c -------------------------------------------------------------------
      subroutine plot_bkg
      open(unit=201,file="bkg.gnu")
       write(201,*)'set terminal png enhanced size 1920,1080'
       write(201,*)"set output 'background.png' "
       write(201,*)'set grid lw 1.2'
       write(201,*)'set border lw 3'
       write(201,*)'unset key'
       write(201,*)'set termoption font ",36"'
       write(201,*)'set xlabel "Energy (eV)"'
       write(201,*)'set ylabel "[{/Symbol m}/{/Symbol r}]"'
       write(201,*)'plot "supp_data.dat" using 1:4 
     &    with lines lw 3 lc rgb "blue",
     &   "supp_data.dat" using 1:5 
     &    with lines lw 3 lc rgb "red" '
       write(201,*)'quit'
       close(201)

       call system('gnuplot bkg.gnu')
       call system('rm bkg.gnu')
       write(*,*)'background spline plotted in "background.png"'

       end
c -------------------------------------------------------------------
      subroutine plot_final_emu
      open(unit=202,file="final_emu.gnu")
       write(202,*)'set terminal png enhanced size 1920,1080'
       write(202,*)"set output 'final_emu.png' "
       write(202,*)'set grid lw 1.2'
       write(202,*)'set border lw 3'
       write(202,*)'unset key'
       write(202,*)'set termoption font ",24"'
       write(202,*)'set xlabel "Energy (eV)"'
       write(201,*)'set ylabel "[{/Symbol m}/{/Symbol r}]"'
       write(202,*)'plot "supp_data.dat" using 1:6:7 
     &    with errorlines lw 3 lc rgb "blue" '
       write(202,*)'quit'
       close(202)

       call system('gnuplot final_emu.gnu')
       call system('rm final_emu.gnu')
       write(*,*)'final E vs mu data plotted in "final_emu.png"'
       end

c -------------------------------------------------------------------
       subroutine plot_chivk(outputname)
       character*99 outputname
       character*128 formt
       integer length

c     Bit of string formatting/preparation
      length=0
      do i=1,99,1
      if(outputname(i:i).NE.' ')length=length+1
      end do
      if(length.LE.9)
     & write(formt,'(A,I1,A)')"(A,A",length,",A)"
      if(length.GT.9)
     & write(formt,'(A,I2,A)')"(A,A",length,",A)"


      open(unit=203,file="chivk.gnu")
       write(203,*)'set terminal png enhanced size 1920,1080'
       write(203,*)"set output 'chivk.png' "
       write(203,*)'set grid lw 1.2'
       write(203,*)'set border lw 3'
       write(203,*)'unset key'
       write(203,*)'set xrange[1:10]'
       write(203,*)'set termoption font ",24"'
       write(203,*)'set multiplot layout 2,1'
       write(203,*)'set xlabel "k (\305^{-1})"'
       write(203,*)'set ylabel "{/Symbol c}"'
       write(203,formt)'plot "', outputname,'.dat" u 1:2:3
     &    with errorbars lw 3 lc rgb "blue" '
       write(203,*)'set xlabel "k (\305^{-1})"'
       write(203,*)'set ylabel "k^2{/Symbol c}"'
       write(203,formt)'plot "', outputname,'.dat" 
     &   using 1:(($2)*($1)**2):(($3)*($1)**2) 
     &    with errorbars lw 3 dt 2 lc rgb "blue" '
       write(203,*)'quit'
       close(203)

       call system('gnuplot chivk.gnu')
       call system('rm chivk.gnu')
       write(*,*)'Chi v k data plotted in "chivk.png"'

       end
c====================================================================

c -------------------------------------------------------------------
      subroutine print_echo
      integer i,j, num_buffer
      double precision x_num_buffer
      character*128 x_text(64)

      i = iffgetsca('&echo_lines',x_num_buffer)
      num_buffer = min(64,int(x_num_buffer))
      write(*,*)'echo buffer=',num_buffer
      do 1010 j = 1, num_buffer
         i = iffgetecho(x_text(j))
         write(*,*)'echo line ', j ,' = ', x_text(j)(1:i)
 1010 continue

      return
      end

c --------------------------------------------------------------------

